#!/s/bin/perl -w

use strict;
use Net::FTP;
use File::Path;

#use Cwd 'abs_path';
use FindBin '$Bin';
my $exedir = $Bin."\/"; 

print STDERR "Downloading Culled PDB lists from PISCES.\n";
my $url = "http://dunbrack.fccc.edu/Guoli/culledpdb/";
mkpath('nmerdb');
chdir("nmerdb");
system("wget wget -r -l1 -q --no-parent -nd -A \"cullpdb_pc30_res3.0_R1.0*.gz\" $url");
system("gunzip -f cullpdb*.gz");

my $PDBDIR = './fraglib';
mkdir "$PDBDIR", 0777 unless -d "$PDBDIR";

my $cullListFile = "";
my @cullFiles = `ls | grep cullpdb`;
foreach my $filegrep (@cullFiles){
	if($filegrep =~ m/cullpdb/){
		if($filegrep !~ m/gz/ && $filegrep !~ m/fasta/){
			$cullListFile = $filegrep;
		}
	}
}

print STDERR "Opening list of PDB entries: $cullListFile.\n";
my %files = ();
open(FILELIST, $cullListFile) || die "cannot open culled file list: $!";
my $line = <FILELIST>;   # skip the first line
while($line = <FILELIST>) {
	my $filename = lc(substr($line,0,4));
	my $chainid = substr($line,4,1);
	if (defined($files{$filename})) {
		$files{$filename} .= ",$chainid";	
	} else {
		$files{$filename} = $chainid;
	}
}
close FILELIST;

print STDERR "Downloading entry files from PDB.\n";

my $host="ftp.wwpdb.org";
my $directory="/pub/pdb/data/structures/all/pdb";

$ENV{FTP_PASSIVE}=1;
my $ftp=Net::FTP->new($host,Timeout=>240) || die "Can't ftp to $host: $!\n";
print STDERR "Connected to RCSB\n";
$ftp->login("anonymous","guest\@guest.com") || die "Can't login to $host: $!\n";
print STDERR "Logged in\n";
$ftp->cwd($directory) || die "Can't cd  $!\n";

# download necessary files
FILE: foreach my $file (keys %files) {
	if (!(-e "$PDBDIR/pdb$file.ent.Z")) {
		#print STDERR "$file\n";
		$ftp->get("pdb$file.ent.gz", "$PDBDIR/pdb$file.ent.gz") || print STDERR "Can't get $file!\n";
	}
}
$ftp->quit;

print STDERR "Unzipping download PDB entries.\n";

system("$exedir/unzipall.sh");

print STDERR "Success! To finish:\n";
print STDERR " (0) [Optional] Remove homologous proteins to your test-set proteins.  This is important if you are benchmarking and developing ACMI.  See README.nmerdb for instructions.\n";
print STDERR " (1) In nmerdb directory, run build_nmer_db to format ACMI's fragment database.  If \$acmi represents the location of your ACMI build and \$cullpdbFile is the non-Fasta culled list downloaded, run:\n";
print STDERR "     % \$acmi/bin/build_nmer_db \$cullpdbFile  nmers.db\n";
print STDERR " (2) Set the environmental variable ACMI_NMERDB to this nmerdb directory in your source file (e.g., ~/.cshrc).\n";

