// //////////////////////////////////////////////////////////////////////////
//
//     build_nmer_db.cpp
//     (c) 2007 Frank DiMaio
//
// //////////////////////////////////////////////////////////////////////////
//
//     This file is part of ACMI.
//
//     ACMI is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 3 of the License, or
//     (at your option) any later version.
//
//     ACMI is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// //////////////////////////////////////////////////////////////////////////

#include "nmer_db.h"

// globals
static string PDBDIR = "../nmerdb/fraglib/";

// returns a lower case version of the string
string tolower (const string & s) {
	string d(s);
	transform (d.begin (), d.end (), d.begin (), (int(*)(int)) tolower);
	return d;
}

//
// build DB using STL constructs
//
int main(int argc, char** argv) {
	srand(1);
	string version = "$HeadURL: file:///u/soni/acmi/svn/tags/releases/1.5.1/utils/build_nmer_db.cpp $";
	int begin = version.find("releases") + 9;
	int end = version.find("utils")-1;

	cerr << "ACMI release version " << version.substr(begin,end-begin) << endl;

	// parse command-line args
	if (argc < 3) {
		cerr << "USAGE: " << argv[0]
		     << " pdb-file-directory output-db-file" << endl;
		exit(1);
	}
	string infilelist = argv[1];
	string outfile = argv[2];

	chem_db_initialize();

	map    < int,  vector< nmerDBEntry > >   DB_nmer;
	vector < string >   DB_filename;
	vector < char > DB_chains;

	ifstream filelist( infilelist.c_str() );
	if (filelist == NULL) { cerr << "ERROR opening " << infilelist << endl; exit(1); }
	string line;

	int n5mers=0, thisIdx=-1;

	getline(filelist, line);  // skip first line
	while( getline(filelist, line) ) {
		string filename = "pdb" + tolower(line.substr(0,4)) + ".pdb";
		char chainid = line[4];

		DB_filename.push_back(filename);
		DB_chains.push_back(chainid);
		thisIdx++;
		cerr << thisIdx << ": " << filename << endl;

		//////////////////////
		//
		// parse pdb file
		ifstream in_pdb( (PDBDIR + "/" + filename).c_str() );
		if (in_pdb == NULL) { cerr << "WARNING: " << filename << " not found!\n"; continue; }

		char conf, lastChain='\0', thisChain, thisResShort;
		string thisRes;
		int thisResId = -10;
		map<int,char> thisPDB;

		while(getline(in_pdb,line)) {
			//if (line.substr(0,3) == "TER") break;

			conf = line[16];

			if (    ( line.substr(0,6) == "ATOM  " || line.substr(0,6) == "HETATM" )
				 &&   line.substr(13,2) == "CA"
				 && ( conf == 'A' || conf == ' ')
				 && ( chainid == '0' || chainid == line[21] ) ) {

				thisRes   = line.substr(17,3);
				if (ThreeLetter2OneLetter.find(thisRes) == ThreeLetter2OneLetter.end()) {
					cerr << "WARNING: unknown residue " << thisRes << " in " << filename << endl;
					thisResShort = 'X';
				} else {
					thisResShort = ThreeLetter2OneLetter[thisRes];
				}

				thisChain = line[21];
				if (thisChain != lastChain) {
					thisResId += 10;   // don't connect adj. chains
					lastChain = thisChain;
				}

				if ( thisPDB.find(thisResId) == thisPDB.end() ) {
					thisPDB[thisResId] = thisResShort;
				} else {
					cerr << "WARNING: duplicate residue " << thisChain << "," << thisResId << endl;
				}
				thisResId++;
			}
		}

		in_pdb.close();

		nmerDBEntry thisEntry;
		thisEntry.fileId = thisIdx;
		thisEntry.chainId = chainid;

		for (map<int,char>::iterator p = thisPDB.begin(); p != thisPDB.end(); ++p) {
			thisEntry.resId = p->first;

			string nmer="_____";
			for (int i=-2; i<=2; ++i) {
				if (thisPDB.find(thisEntry.resId+i) != thisPDB.end()) {
					nmer[i+2] = thisPDB[thisEntry.resId+i];
				}
			}

			//cerr << nmer << endl;
			int nmerhash = chem_db_hashcode(nmer);
			if ( DB_nmer.find(nmerhash) == DB_nmer.end() ) {
				DB_nmer[nmerhash] = vector< nmerDBEntry >();
			}
			DB_nmer[nmerhash].push_back( thisEntry );

			n5mers++;
		}
	}

	// dump to disk
	int nchains=DB_chains.size();
	DB_dump(outfile, DB_nmer , DB_filename);

	cerr << "Found " << n5mers << " 5mers in " << nchains << " chains\n";
}
