/*
 * mem_usage.cpp
 *
 *  Created on: Nov 11, 2010
 *      Author: soni
 */
#include "../src/BP.h"
#include "getopts.h"   // parse command line args

int main(int argc, char** argv) {
	version();
	///
	/// read input files
	Options bpOpts;
  	int switchInt;

	//                  Short   Long        Description           Take args?
  	bpOpts.addOption("w",    "working",      "[REQUIRED] Working Directory",      true);
  	bpOpts.addOption("c",    "cryst",        "[REQUIRED] Input crystal file",      true);
	bpOpts.addOption("s",    "seq",          "[REQUIRED] Input sequence file",      true);

	string crystfile, seqfile, workingdir;

	bpOpts.parse(argc, argv);
	while ((switchInt = bpOpts.cycle()) >= 0) {
		vector<string> pargs;
		Crystal nullCryst;
		switch(switchInt) {
			case 0:   // Option 'working-dir'
				workingdir =  bpOpts.getArgs(switchInt); break;
			case 1:   // Option 'cryst'
				crystfile =  bpOpts.getArgs(switchInt); break;
			case 2:   // Option 'seq'
				seqfile =  bpOpts.getArgs(switchInt); break;
		}
	}
	// at a minimum, need to specify workingdir, map, cryst, seq
	if (workingdir.length() == 0 || crystfile.length() == 0 || seqfile.length() == 0) {
		cerr << "Error: Must specify a working directory, crystal file, and sequence file.\n";
		cerr << "       See \'" << argv[0] << " -h\' for more details\n";
		exit(1);
	}
	string matchFormatString = workingdir + "/priors/";
	////////////////////
	//
	// read input files
	//
	Crystal crystData(crystfile);
	Sequence  seqData(seqfile);
	int nResSum = seqData.nres();
	vec3<int> dims = vec3<int>(0,0,0);;
	for (int i=0; i<nResSum; i++) {
		if (seqData.serialGet(i) == '_') continue;  // gap

		string mer = seqData.serialGet5mer(i);
		dims = Belief::getDims(matchFormatString, mer);
		if(dims[0]*dims[1]*dims[2] == 0) continue; //repeat due to failure

		break;
	}
	crystData.buildMaps(dims);

	double approxMemUsage = ( (nResSum*36.0) * crystData.getPtrMapInv().size() ) / (1024.0*1024.0) + 100.0;
	cerr << "Approximate memory needed: " << approxMemUsage << "MB\n";
}
